@@ -142,10 +142,6 @@ typedef struct _cl_fmap_item {
 *	p_up
 *		Pointer to the map item that is the parent of the node.
 *
-*	p_nil
-*		Pointer to the map's NIL item, used as a terminator for leaves.
-*		The NIL sentinel is in the cl_fmap_t structure.
-*
 *	color
 *		Indicates whether a node is red or black in the map.
 *
@@ -737,6 +733,10 @@ cl_fmap_remove_item(IN cl_fmap_t * const p_map,
 		    IN cl_fmap_item_t * const p_item);
 /*
 * PARAMETERS
+*	p_map
+*		[in] Pointer to a cl_fmap_t structure from which to
+*		remove item.
+*
 *	p_item
 *		[in] Pointer to a map item to remove from its flexi map.
 *
