@@ -61,7 +61,6 @@
 #include <opensm/osm_port.h>
 #include <opensm/osm_db.h>
 #include <opensm/osm_remote_sm.h>
-#include <opensm/osm_multicast.h>
 
 #ifdef __cplusplus
 #  define BEGIN_C_DECLS extern "C" {
@@ -402,8 +401,8 @@ ib_api_status_t osm_sm_bind(IN osm_sm_t * p_sm, IN ib_net64_t port_guid);
 */
 ib_api_status_t osm_req_get(IN osm_sm_t * sm, IN const osm_dr_path_t * p_path,
 			    IN ib_net16_t attr_id, IN ib_net32_t attr_mod,
-			    IN boolean_t find_mkey, ib_net64_t m_key,
-			    IN cl_disp_msgid_t err_msg,
+			    IN boolean_t find_mkey, IN ib_net64_t m_key,
+			    IN uint32_t timeout, IN cl_disp_msgid_t err_msg,
 			    IN const osm_madw_context_t * p_context);
 /*
 * PARAMETERS
@@ -427,6 +426,9 @@ ib_api_status_t osm_req_get(IN osm_sm_t * sm, IN const osm_dr_path_t * p_path,
 * 		[in] M_Key value to be send with this MAD. Applied, only when
 * 		     find_mkey is FALSE.
 *
+*	timeout
+*		[in] Transaction timeout in msec.
+*
 *	err_msg
 *		[in] Message id with which to post this MAD if an error occurs.
 *
@@ -476,7 +478,8 @@ osm_madw_t *osm_prepare_req_set(IN osm_sm_t * sm, IN const osm_dr_path_t * p_pat
 				IN const uint8_t * p_payload,
 				IN size_t payload_size, IN ib_net16_t attr_id,
 				IN ib_net32_t attr_mod, IN boolean_t find_mkey,
-				IN ib_net64_t m_key, IN cl_disp_msgid_t err_msg,
+				IN ib_net64_t m_key, IN uint32_t timeout,
+				IN cl_disp_msgid_t err_msg,
 				IN const osm_madw_context_t * p_context);
 /*
 * PARAMETERS
@@ -505,6 +508,9 @@ osm_madw_t *osm_prepare_req_set(IN osm_sm_t * sm, IN const osm_dr_path_t * p_pat
 * 		[in] M_Key value to be send with this MAD. Applied, only when
 * 		     find_mkey is FALSE.
 *
+*	timeout
+*		[in] Transaction timeout in msec.
+*
 *	err_msg
 *		[in] Message id with which to post this MAD if an error occurs.
 *
@@ -530,7 +536,8 @@ ib_api_status_t osm_req_set(IN osm_sm_t * sm, IN const osm_dr_path_t * p_path,
 			    IN const uint8_t * p_payload,
 			    IN size_t payload_size, IN ib_net16_t attr_id,
 			    IN ib_net32_t attr_mod, IN boolean_t find_mkey,
-			    IN ib_net64_t m_key, IN cl_disp_msgid_t err_msg,
+			    IN ib_net64_t m_key, IN uint32_t timeout,
+			    IN cl_disp_msgid_t err_msg,
 			    IN const osm_madw_context_t * p_context);
 /*
 * PARAMETERS
@@ -560,6 +567,9 @@ ib_api_status_t osm_req_set(IN osm_sm_t * sm, IN const osm_dr_path_t * p_path,
 * 		[in] M_Key value to be send with this MAD. Applied, only when
 * 		     find_mkey is FALSE.
 *
+*	timeout
+*		[in] Transaction timeout in msec.
+*
 *	err_msg
 *		[in] Message id with which to post this MAD if an error occurs.
 *
