@@ -5,6 +5,7 @@
  * Copyright (c) 2009 HNR Consulting. All rights reserved.
  * Copyright (c) 2009 System Fabric Works, Inc. All rights reserved.
  * Copyright (c) 2009-2011 ZIH, TU Dresden, Federal Republic of Germany. All rights reserved.
+ * Copyright (C) 2012-2017 Tokyo Institute of Technology. All rights reserved.
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -160,6 +161,9 @@ static void show_usage(void)
 	       "          This will effect the handover cases, where master\n"
 	       "          is chosen by priority and GUID.  Range goes\n"
 	       "          from 0 (lowest priority) to 15 (highest).\n\n");
+	printf("--subnet_prefix <prefix>\n"
+	       "          Set the subnet prefix to something other than the\n"
+	       "          default value of 0xfe80000000000000\n\n");
 	printf("--smkey, -k <SM_Key>\n"
 	       "          This option specifies the SM's SM_Key (64 bits).\n"
 	       "          This will effect SM authentication.\n"
@@ -181,7 +185,8 @@ static void show_usage(void)
 	       "          If all configured routing engines fail, OpenSM will always\n"
 	       "          attempt to route with Min Hop unless 'no_fallback' is\n"
 	       "          included in the list of routing engines.\n"
-	       "          Supported engines: updn, dnup, file, ftree, lash, dor, torus-2QoS, dfsssp, sssp\n\n");
+	       "          Supported engines: updn, dnup, file, ftree, lash, dor,\n"
+	       "                             torus-2QoS, nue, dfsssp, sssp\n\n");
 	printf("--do_mesh_analysis\n"
 	       "          This option enables additional analysis for the lash\n"
 	       "          routing engine to precondition switch port assignments\n"
@@ -189,9 +194,14 @@ static void show_usage(void)
 	       "          of SLs required to give a deadlock free routing\n\n");
 	printf("--lash_start_vl <vl number>\n"
 	       "          Sets the starting VL to use for the lash routing algorithm.\n"
-	       "          Defaults to 0.\n");
+	       "          Defaults to 0.\n\n");
 	printf("--sm_sl <sl number>\n"
 	       "          Sets the SL to use to communicate with the SM/SA. Defaults to 0.\n\n");
+	printf("--nue_max_num_vls <vl number>\n"
+	       "          Sets the maximum number of VLs to be used by Nue routing.\n"
+	       "          Defaults to 1 to enforce deadlock-freedom even if QoS is not\n"
+	       "          enabled. Set to 0 if Nue should automatically determine and\n"
+	       "          choose maximum supported by the fabric, or any integer >= 1.\n\n");
 	printf("--connect_roots, -z\n"
 	       "          This option enforces routing engines (up/down and \n"
 	       "          fat-tree) to make connectivity between root switches\n"
@@ -237,7 +247,7 @@ static void show_usage(void)
 	       "          If zero disable (default), otherwise use the value as a random seed\n\n");
 	printf("--max_reverse_hops, -H <hop_count>\n"
 	       "          Set the max number of hops the wrong way around\n"
-	       "          an I/O node is allowed to do (connectivity for I/O nodes on top swithces)\n\n");
+	       "          an I/O node is allowed to do (connectivity for I/O nodes on top switches)\n\n");
 	printf("--ids_guid_file, -m <path to file>\n"
 	       "          Name of the map file with set of the IDs which will be used\n"
 	       "          by Up/Down routing algorithm instead of node GUIDs\n"
@@ -312,6 +322,8 @@ static void show_usage(void)
 	       "          This option forces OpenSM to honor the guid2lid file,\n"
 	       "          when it comes out of Standby state, if such file exists\n"
 	       "          under OSM_CACHE_DIR, and is valid. By default, this is FALSE.\n\n");
+	printf("--dump_files_dir <directory-name>"
+	       "          The directory to hold the file dumps.\n");
 	printf("--log_file, -f <log-file-name>\n"
 	       "          This option defines the log to be the given file.\n"
 	       "          By default, the log goes to /var/log/opensm.log.\n"
@@ -492,7 +504,7 @@ static ib_net64_t get_port_guid(IN osm_opensm_t * p_osm, uint64_t port_guid)
 		fflush(stdout);
 		if (scanf("%u", &choice) <= 0) {
 			char junk[128];
-			if (scanf("%s", junk) <= 0)
+			if (scanf("%127s", junk) <= 0)
 				printf("\nError: Cannot scan!\n");
 		} else if (choice == 0)
 			return 0;
@@ -558,6 +570,8 @@ static int daemonize(osm_opensm_t * osm)
 	dup2(fd, 1);
 	dup2(fd, 2);
 
+	close(fd);
+
 	return 0;
 }
 
@@ -656,6 +670,7 @@ int main(int argc, char *argv[])
 		{"once", 0, NULL, 'o'},
 		{"reassign_lids", 0, NULL, 'r'},
 		{"priority", 1, NULL, 'p'},
+		{"subnet_prefix", 1, NULL, 16},
 		{"smkey", 1, NULL, 'k'},
 		{"routing_engine", 1, NULL, 'R'},
 		{"ucast_cache", 0, NULL, 'A'},
@@ -692,6 +707,8 @@ int main(int argc, char *argv[])
 		{"log_prefix", 1, NULL, 9},
 		{"torus_config", 1, NULL, 10},
 		{"guid_routing_order_no_scatter", 0, NULL, 13},
+		{"nue_max_num_vls", 1, NULL, 15},
+		{"dump_files_dir", 1, NULL, 17},
 		{NULL, 0, NULL, 0}	/* Required at the end of the array */
 	};
 
@@ -998,6 +1015,11 @@ int main(int argc, char *argv[])
 			printf(" Priority = %d\n", temp);
 			break;
 
+		case 16:
+			opt.subnet_prefix = cl_hton64(strtoull(optarg, NULL, 16));
+			printf(" Subnet_Prefix = <0x%" PRIx64 ">\n", cl_hton64(opt.subnet_prefix));
+			break;
+
 		case 'k':
 			sm_key = cl_hton64(strtoull(optarg, NULL, 16));
 			printf(" SM Key <0x%" PRIx64 ">\n", cl_hton64(sm_key));
@@ -1141,6 +1163,20 @@ int main(int argc, char *argv[])
 		case 13:
 			opt.guid_routing_order_no_scatter = TRUE;
 			break;
+		case 15:
+			temp = strtoul(optarg, NULL, 0);
+			if (temp >= IB_MAX_NUM_VLS) {
+				fprintf(stderr,
+					"ERROR: maximum #VLs for nue routing must be between 0 and %d\n",
+					IB_MAX_NUM_VLS);
+				return -1;
+			}
+			opt.nue_max_num_vls = (uint8_t) temp;
+			printf(" Nue maximum #VLs = %d\n", opt.nue_max_num_vls);
+			break;
+		case 17:
+			SET_STR_OPT(opt.dump_files_dir, optarg);
+			break;
 		case 'h':
 		case '?':
 		case ':':
@@ -1182,7 +1218,10 @@ int main(int argc, char *argv[])
 		daemonize(&osm);
 	}
 
-	complib_init();
+	if (complib_init_v2() != CL_SUCCESS) {
+		printf("\ncomplib_init_v2 error\n");
+		return -1;
+	}
 
 	status = osm_opensm_init(&osm, &opt);
 	if (status != IB_SUCCESS) {
