@@ -127,7 +127,7 @@ static void cc_setup_mad_data(osm_sm_t * p_sm)
 	osm_congestion_control_t *p_cc = &p_sm->p_subn->p_osm->cc;
 	osm_subn_opt_t *p_opt = &p_sm->p_subn->opt;
 	uint16_t ccti_limit;
-	int i;
+	unsigned i;
 
 	/* Switch Congestion Setting */
 	p_cc->sw_cong_setting.control_map = p_opt->cc_sw_cong_setting_control_map;
@@ -479,14 +479,15 @@ static void cc_rcv_mad(void *context, void *data)
 	p_port->cc_timeout_count = 0;
 
 	if (p_cc_mad->header.status) {
-		if (p_cc_mad->header.status & IB_MAD_STATUS_UNSUP_METHOD
+		if (p_cc_mad->header.status & IB_MAD_STATUS_UNSUP_CLASS_VER
+		    || p_cc_mad->header.status & IB_MAD_STATUS_UNSUP_METHOD
 		    || p_cc_mad->header.status & IB_MAD_STATUS_UNSUP_METHOD_ATTR)
-			p_port->cc_unavailable_flag = 1;
+			p_port->cc_unavailable_flag = TRUE;
 		cl_plock_release(&p_osm->lock);
 		goto Exit;
 	}
 	else
-		p_port->cc_unavailable_flag = 0;
+		p_port->cc_unavailable_flag = FALSE;
 
 	if (p_cc_mad->header.attr_id == IB_MAD_ATTR_SW_CONG_SETTING) {
 		ib_sw_cong_setting_t *p_sw_cong_setting;
@@ -704,8 +705,10 @@ static void cc_mad_send_err_callback(void *bind_context,
 	if (p_madw->status == IB_TIMEOUT) {
 		p_port->cc_timeout_count++;
 		if (p_port->cc_timeout_count > OSM_CC_TIMEOUT_COUNT_THRESHOLD
-		    && !p_port->cc_unavailable_flag)
-			p_port->cc_unavailable_flag++;
+		    && !p_port->cc_unavailable_flag) {
+			p_port->cc_unavailable_flag = TRUE;
+			p_port->cc_timeout_count = 0;
+		}
 	} else
 		p_cc->subn->subnet_initialization_error = TRUE;
 
