@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2004-2009 Voltaire, Inc. All rights reserved.
- * Copyright (c) 2002-2006 Mellanox Technologies LTD. All rights reserved.
+ * Copyright (c) 2002-2015 Mellanox Technologies LTD. All rights reserved.
  * Copyright (c) 1996-2003 Intel Corporation. All rights reserved.
  * Copyright (c) 2009 HNR Consulting. All rights reserved.
  * Copyright (c) 2013 Oracle and/or its affiliates. All rights reserved.
@@ -104,6 +104,8 @@ static cl_status_t match_inf_rec(IN const cl_list_item_t * p_list_item,
 {
 	osm_infr_t *p_infr_rec = (osm_infr_t *) context;
 	osm_infr_t *p_infr = (osm_infr_t *) p_list_item;
+	ib_inform_info_t *p_ii_rec = &p_infr_rec->inform_record.inform_info;
+	ib_inform_info_t *p_ii = &p_infr->inform_record.inform_info;
 	osm_log_t *p_log = p_infr_rec->sa->p_log;
 	cl_status_t status = CL_NOT_FOUND;
 
@@ -116,86 +118,66 @@ static cl_status_t match_inf_rec(IN const cl_list_item_t * p_list_item,
 	}
 
 	/* if inform_info.gid is not zero, ignore lid range */
-	if (ib_gid_is_notzero(&p_infr_rec->inform_record.inform_info.gid)) {
-		if (memcmp(&p_infr->inform_record.inform_info.gid,
-			   &p_infr_rec->inform_record.inform_info.gid,
-			   sizeof(p_infr->inform_record.inform_info.gid))) {
+	if (ib_gid_is_notzero(&p_ii_rec->gid)) {
+		if (memcmp(&p_ii->gid, &p_ii_rec->gid, sizeof(p_ii->gid))) {
 			OSM_LOG(p_log, OSM_LOG_DEBUG,
 				"Differ by InformInfo.gid\n");
 			goto Exit;
 		}
 	} else {
-		if ((p_infr->inform_record.inform_info.lid_range_begin !=
-		     p_infr_rec->inform_record.inform_info.lid_range_begin) ||
-		    (p_infr->inform_record.inform_info.lid_range_end !=
-		     p_infr_rec->inform_record.inform_info.lid_range_end)) {
+		if ((p_ii->lid_range_begin != p_ii_rec->lid_range_begin) ||
+		    (p_ii->lid_range_end != p_ii_rec->lid_range_end)) {
 			OSM_LOG(p_log, OSM_LOG_DEBUG,
 				"Differ by InformInfo.LIDRange\n");
 			goto Exit;
 		}
 	}
 
-	if (p_infr->inform_record.inform_info.trap_type !=
-	    p_infr_rec->inform_record.inform_info.trap_type) {
+	if (p_ii->trap_type != p_ii_rec->trap_type) {
 		OSM_LOG(p_log, OSM_LOG_DEBUG,
 			"Differ by InformInfo.TrapType\n");
 		goto Exit;
 	}
 
-	if (p_infr->inform_record.inform_info.is_generic !=
-	    p_infr_rec->inform_record.inform_info.is_generic) {
+	if (p_ii->is_generic != p_ii_rec->is_generic) {
 		OSM_LOG(p_log, OSM_LOG_DEBUG,
 			"Differ by InformInfo.IsGeneric\n");
 		goto Exit;
 	}
 
-	if (p_infr->inform_record.inform_info.is_generic) {
-		if (p_infr->inform_record.inform_info.g_or_v.generic.trap_num !=
-		    p_infr_rec->inform_record.inform_info.g_or_v.generic.
-		    trap_num)
+	if (p_ii->is_generic) {
+		if (p_ii->g_or_v.generic.trap_num !=
+		    p_ii_rec->g_or_v.generic.trap_num)
 			OSM_LOG(p_log, OSM_LOG_DEBUG,
 				"Differ by InformInfo.Generic.TrapNumber\n");
-		else if (p_infr->inform_record.inform_info.g_or_v.generic.
-			 qpn_resp_time_val !=
-			 p_infr_rec->inform_record.inform_info.g_or_v.generic.
-			 qpn_resp_time_val)
+		else if (p_ii->g_or_v.generic.qpn_resp_time_val !=
+			 p_ii_rec->g_or_v.generic.qpn_resp_time_val)
 			OSM_LOG(p_log, OSM_LOG_DEBUG,
 				"Differ by InformInfo.Generic.QPNRespTimeVal\n");
-		else if (p_infr->inform_record.inform_info.g_or_v.generic.
-			 node_type_msb !=
-			 p_infr_rec->inform_record.inform_info.g_or_v.generic.
-			 node_type_msb)
+		else if (p_ii->g_or_v.generic.node_type_msb !=
+			 p_ii_rec->g_or_v.generic.node_type_msb)
 			OSM_LOG(p_log, OSM_LOG_DEBUG,
 				"Differ by InformInfo.Generic.NodeTypeMSB\n");
-		else if (p_infr->inform_record.inform_info.g_or_v.generic.
-			 node_type_lsb !=
-			 p_infr_rec->inform_record.inform_info.g_or_v.generic.
-			 node_type_lsb)
+		else if (p_ii->g_or_v.generic.node_type_lsb !=
+			 p_ii_rec->g_or_v.generic.node_type_lsb)
 			OSM_LOG(p_log, OSM_LOG_DEBUG,
 				"Differ by InformInfo.Generic.NodeTypeLSB\n");
 		else
 			status = CL_SUCCESS;
 	} else {
-		if (p_infr->inform_record.inform_info.g_or_v.vend.dev_id !=
-		    p_infr_rec->inform_record.inform_info.g_or_v.vend.dev_id)
+		if (p_ii->g_or_v.vend.dev_id != p_ii_rec->g_or_v.vend.dev_id)
 			OSM_LOG(p_log, OSM_LOG_DEBUG,
 				"Differ by InformInfo.Vendor.DeviceID\n");
-		else if (p_infr->inform_record.inform_info.g_or_v.vend.
-			 qpn_resp_time_val !=
-			 p_infr_rec->inform_record.inform_info.g_or_v.vend.
-			 qpn_resp_time_val)
+		else if (p_ii->g_or_v.vend.qpn_resp_time_val !=
+			 p_ii_rec->g_or_v.vend.qpn_resp_time_val)
 			OSM_LOG(p_log, OSM_LOG_DEBUG,
 				"Differ by InformInfo.Vendor.QPNRespTimeVal\n");
-		else if (p_infr->inform_record.inform_info.g_or_v.vend.
-			 vendor_id_msb !=
-			 p_infr_rec->inform_record.inform_info.g_or_v.vend.
-			 vendor_id_msb)
+		else if (p_ii->g_or_v.vend.vendor_id_msb !=
+			 p_ii_rec->g_or_v.vend.vendor_id_msb)
 			OSM_LOG(p_log, OSM_LOG_DEBUG,
 				"Differ by InformInfo.Vendor.VendorIdMSB\n");
-		else if (p_infr->inform_record.inform_info.g_or_v.vend.
-			 vendor_id_lsb !=
-			 p_infr_rec->inform_record.inform_info.g_or_v.vend.
-			 vendor_id_lsb)
+		else if (p_ii->g_or_v.vend.vendor_id_lsb !=
+			 p_ii_rec->g_or_v.vend.vendor_id_lsb)
 			OSM_LOG(p_log, OSM_LOG_DEBUG,
 				"Differ by InformInfo.Vendor.VendorIdLSB\n");
 		else
@@ -397,11 +379,12 @@ static int is_access_permitted(osm_infr_t *p_infr_rec,
 	osm_port_t *p_src_port;
 	osm_port_t *p_dest_port;
 
-	/* In case of GID_IN(64) or GID_OUT(65) traps the source gid
-	   comparison should be done on the trap source (saved as the gid in the
-	   data details field).
-	   For traps MC_CREATE(66) or MC_DELETE(67) the data details gid is
-	   the MGID. We need to check whether the subscriber has a compatible
+	/* In case of SM_GID_IN_SERVICE_TRAP(64) or SM_GID_OUT_OF_SERVICE_TRAP(65) traps
+	   the source gid comparison should be done on the trap source (saved
+	   as the gid in the data details field).
+	   For traps SM_MGID_CREATED_TRAP(66) or SM_MGID_DESTROYED_TRAP(67)
+	   the data details gid is the MGID.
+	   We need to check whether the subscriber has a compatible
 	   pkey with MC group.
 	   In all other cases the issuer gid is the trap source.
 	*/
