@@ -1,5 +1,6 @@
 /*
  * Copyright (c) 2006-2009 Voltaire, Inc. All rights reserved.
+ * Copyright (c) 2010-2015 Mellanox Technologies LTD. All rights reserved.
  * Copyright (c) 2009 HNR Consulting. All rights reserved.
  *
  * This software is available to you under a choice of one of two
@@ -93,6 +94,7 @@ static qos_mad_item_t *osm_qos_mad_create(IN osm_sm_t * sm,
 	osm_node_t *p_node;
 	osm_physp_t *physp0;
 	ib_net64_t m_key;
+	uint32_t timeout = 0;
 
 	p_node = osm_physp_get_node_ptr(p);
 	if (osm_node_get_type(p_node) == IB_NODE_TYPE_SWITCH &&
@@ -107,6 +109,8 @@ static qos_mad_item_t *osm_qos_mad_create(IN osm_sm_t * sm,
 		context.slvl_context.node_guid = osm_node_get_node_guid(p_node);
 		context.slvl_context.port_guid = osm_physp_get_port_guid(p);
 		context.slvl_context.set_method = TRUE;
+		if ((attr_mod & 0x30000) != 0)	/* optimized ? */
+			timeout = sm->p_subn->opt.long_transaction_timeout;
 		break;
 	case IB_MAD_ATTR_VL_ARBITRATION:
 		context.vla_context.node_guid = osm_node_get_node_guid(p_node);
@@ -127,7 +131,7 @@ static qos_mad_item_t *osm_qos_mad_create(IN osm_sm_t * sm,
 				     p_data, data_size,
 				     attr_id, cl_hton32(attr_mod),
 				     FALSE, m_key,
-				     CL_DISP_MSGID_NONE, &context);
+				     timeout, CL_DISP_MSGID_NONE, &context);
 
 	if (p_madw == NULL) {
 		free(p_mad);
@@ -204,6 +208,10 @@ static ib_api_status_t vlarb_update(osm_sm_t * sm, osm_physp_t * p,
 	ib_port_info_t *p_pi = &p->port_info;
 	unsigned len;
 
+	if (ib_port_info_get_vl_cap(p_pi) == 1) {	/* no VLArb if 1 data VL */
+		p->vl_high_limit = 0;
+		return status;
+	}
 	if (p_pi->vl_arb_low_cap > 0) {
 		len = p_pi->vl_arb_low_cap < IB_NUM_VL_ARB_ELEMENTS_IN_BLOCK ?
 		    p_pi->vl_arb_low_cap : IB_NUM_VL_ARB_ELEMENTS_IN_BLOCK;
@@ -312,6 +320,8 @@ static int qos_extports_setup(osm_sm_t * sm, osm_node_t *node,
 
 	for (out = 1; out < num_ports; out++) {
 		p = osm_node_get_physp_ptr(node, out);
+		if (!p)
+			continue;
 		if (ib_port_info_get_port_state(&p->port_info) == IB_LINK_DOWN)
 			continue;
 		force_update = p->need_update || sm->p_subn->need_update;
@@ -334,6 +344,8 @@ static int qos_extports_setup(osm_sm_t * sm, osm_node_t *node,
 
 		for (out = 1; out < num_ports; out++) {
 			p = osm_node_get_physp_ptr(node, out);
+			if (!p)
+				continue;
 			if (ib_port_info_get_port_state(&p->port_info) ==
 			    IB_LINK_DOWN)
 				continue;
@@ -347,6 +359,8 @@ static int qos_extports_setup(osm_sm_t * sm, osm_node_t *node,
 			}
 
 		}
+		if (!p0)
+			return -1;
 		force_update = node->sw->need_update || sm->p_subn->need_update;
 		if (sl2vl_update_table(sm, p0, p0->port_num, 0x30000, force_update,
 					&qcfg->sl2vl, port_mad_list))
@@ -357,6 +371,8 @@ static int qos_extports_setup(osm_sm_t * sm, osm_node_t *node,
 		 */
 		for (out = 1; out < num_ports; out++) {
 			p = osm_node_get_physp_ptr(node, out);
+			if (!p)
+				continue;
 			if (ib_port_info_get_port_state(&p->port_info) ==
 			    IB_LINK_DOWN)
 				continue;
@@ -376,6 +392,8 @@ static int qos_extports_setup(osm_sm_t * sm, osm_node_t *node,
 	out = ib_switch_info_is_enhanced_port0(&node->sw->switch_info) ? 0 : 1;
 	for (; out < num_ports; out++) {
 		p = osm_node_get_physp_ptr(node, out);
+		if (!p)
+			continue;
 		if (ib_port_info_get_port_state(&p->port_info) == IB_LINK_DOWN)
 			continue;
 		force_update = p->need_update || sm->p_subn->need_update;
@@ -472,8 +490,10 @@ int osm_qos_setup(osm_opensm_t * p_osm)
 		p_next = cl_qmap_next(p_next);
 
 		p_list = (qos_mad_list_t *) malloc(sizeof(*p_list));
-		if (!p_list)
+		if (!p_list) {
+			cl_plock_release(&p_osm->lock);
 			return -1;
+		}
 
 		memset(p_list, 0, sizeof(*p_list));
 
@@ -482,8 +502,10 @@ int osm_qos_setup(osm_opensm_t * p_osm)
 		p_node = p_port->p_node;
 		if (p_node->sw) {
 			if (qos_extports_setup(&p_osm->sm, p_node, &swe_config,
-					       &p_list->port_mad_list))
+					       &p_list->port_mad_list)) {
+				cl_plock_release(&p_osm->lock);
 				ret = -1;
+			}
 
 			/* skip base port 0 */
 			if (!ib_switch_info_is_enhanced_port0
@@ -503,9 +525,10 @@ int osm_qos_setup(osm_opensm_t * p_osm)
 			cfg = &ca_config;
 
 		if (qos_endport_setup(&p_osm->sm, p_port->p_physp, cfg,
-				      vlarb_only, &p_list->port_mad_list))
-
+				      vlarb_only, &p_list->port_mad_list)) {
+			cl_plock_release(&p_osm->lock);
 			ret = -1;
+		}
 Continue:
 		/* if MAD list is not empty, add it to the global MAD list */
 		if (cl_qlist_count(&p_list->port_mad_list)) {
