@@ -146,7 +146,7 @@ static void state_mgr_get_sw_info(IN cl_map_item_t * p_object, IN void *context)
 
 	status = osm_req_get(sm, p_dr_path, IB_MAD_ATTR_SWITCH_INFO, 0,
 			     FALSE, ib_port_info_get_m_key(&p_physp->port_info),
-			     OSM_MSG_LIGHT_SWEEP_FAIL, &mad_context);
+			     0, OSM_MSG_LIGHT_SWEEP_FAIL, &mad_context);
 	if (status != IB_SUCCESS)
 		OSM_LOG(sm->p_log, OSM_LOG_ERROR, "ERR 3304: "
 			"Request for SwitchInfo from 0x%" PRIx64 " failed (%s)\n",
@@ -193,7 +193,7 @@ static void state_mgr_get_remote_port_info(IN osm_sm_t * sm,
 	/* note that with some negative logic - if the query failed it means
 	 * that there is no point in going to heavy sweep */
 	status = osm_req_get(sm, &rem_node_dr_path, IB_MAD_ATTR_PORT_INFO, 0,
-			     TRUE, 0, CL_DISP_MSGID_NONE, &mad_context);
+			     TRUE, 0, 0, CL_DISP_MSGID_NONE, &mad_context);
 	if (status != IB_SUCCESS)
 		OSM_LOG(sm->p_log, OSM_LOG_ERROR, "ERR 332E: "
 			"Request for remote PortInfo from 0x%" PRIx64 " failed (%s)\n",
@@ -253,7 +253,7 @@ static ib_api_status_t state_mgr_sweep_hop_0(IN osm_sm_t * sm)
 		osm_dr_path_init(&dr_path, 0, path_array);
 		CL_PLOCK_ACQUIRE(sm->p_lock);
 		status = osm_req_get(sm, &dr_path, IB_MAD_ATTR_NODE_INFO, 0,
-				     TRUE, 0, CL_DISP_MSGID_NONE, NULL);
+				     TRUE, 0, 0, CL_DISP_MSGID_NONE, NULL);
 		CL_PLOCK_RELEASE(sm->p_lock);
 		if (status != IB_SUCCESS)
 			OSM_LOG(sm->p_log, OSM_LOG_ERROR, "ERR 3305: "
@@ -554,7 +554,7 @@ static ib_api_status_t state_mgr_sweep_hop_1(IN osm_sm_t * sm)
 		osm_dr_path_init(&hop_1_path, 1, path_array);
 		CL_PLOCK_ACQUIRE(sm->p_lock);
 		status = osm_req_get(sm, &hop_1_path, IB_MAD_ATTR_NODE_INFO, 0,
-				     TRUE, 0, CL_DISP_MSGID_NONE, &context);
+				     TRUE, 0, 0, CL_DISP_MSGID_NONE, &context);
 		CL_PLOCK_RELEASE(sm->p_lock);
 		if (status != IB_SUCCESS)
 			OSM_LOG(sm->p_log, OSM_LOG_ERROR, "ERR 3311: "
@@ -586,7 +586,7 @@ static ib_api_status_t state_mgr_sweep_hop_1(IN osm_sm_t * sm)
 				CL_PLOCK_ACQUIRE(sm->p_lock);
 				status = osm_req_get(sm, &hop_1_path,
 						     IB_MAD_ATTR_NODE_INFO, 0,
-						     TRUE, 0,
+						     TRUE, 0, 0,
 						     CL_DISP_MSGID_NONE,
 						     &context);
 				CL_PLOCK_RELEASE(sm->p_lock);
@@ -632,7 +632,7 @@ static void query_sm_info(cl_map_item_t * item, void *cxt)
 	ret = osm_req_get(sm, osm_physp_get_dr_path_ptr(p_port->p_physp),
 			  IB_MAD_ATTR_SM_INFO, 0, FALSE,
 			  ib_port_info_get_m_key(&p_port->p_physp->port_info),
-			  CL_DISP_MSGID_NONE, &context);
+			  0, CL_DISP_MSGID_NONE, &context);
 	if (ret != IB_SUCCESS)
 		OSM_LOG(sm->p_log, OSM_LOG_ERROR, "ERR 3314: "
 			"Failure requesting SMInfo (%s)\n",
@@ -682,7 +682,7 @@ static void state_mgr_reset_state_change_bit(IN cl_map_item_t * obj,
 			     sizeof(si), IB_MAD_ATTR_SWITCH_INFO,
 			     0, FALSE,
 			     ib_port_info_get_m_key(&p_physp->port_info),
-			     CL_DISP_MSGID_NONE, &mad_context);
+			     0, CL_DISP_MSGID_NONE, &mad_context);
 
 	if (status != IB_SUCCESS)
 		OSM_LOG(sm->p_log, OSM_LOG_ERROR, "ERR 332A: "
@@ -727,7 +727,7 @@ static void state_mgr_update_node_desc(IN cl_map_item_t * obj, IN void *context)
 	status = osm_req_get(sm, osm_physp_get_dr_path_ptr(p_physp),
 			     IB_MAD_ATTR_NODE_DESC, 0, FALSE,
 			     ib_port_info_get_m_key(&p_physp->port_info),
-			     CL_DISP_MSGID_NONE, &mad_context);
+			     0, CL_DISP_MSGID_NONE, &mad_context);
 	if (status != IB_SUCCESS)
 		OSM_LOG(sm->p_log, OSM_LOG_ERROR,
 			"ERR 331B: Failure initiating NodeDescription request "
@@ -998,27 +998,14 @@ static void state_mgr_send_handover(IN osm_sm_t * sm, IN osm_remote_sm_t * p_sm)
 	p_smi->act_count = cl_hton32(sm->p_subn->p_osm->stats.qp0_mads_sent);
 	p_smi->pri_state = (uint8_t) (sm->p_subn->sm_state |
 				      sm->p_subn->opt.sm_priority << 4);
-	/*
-	 * Return 0 for the SM key unless we authenticate the requester
-	 * as the master SM.
-	 */
-	if (ib_sminfo_get_state(&p_sm->smi) == IB_SMINFO_STATE_MASTER) {
-		OSM_LOG(sm->p_log, OSM_LOG_DEBUG,
-			"Responding to master SM with real sm_key\n");
-		p_smi->sm_key = sm->p_subn->opt.sm_key;
-	} else {
-		/* The requester is not authenticated as master - set sm_key to zero */
-		OSM_LOG(sm->p_log, OSM_LOG_DEBUG,
-			"Responding to SM not master with zero sm_key\n");
-		p_smi->sm_key = 0;
-	}
+	p_smi->sm_key = sm->p_subn->opt.sm_key;
 
 	CL_PLOCK_ACQUIRE(sm->p_lock);
 	status = osm_req_set(sm, osm_physp_get_dr_path_ptr(p_port->p_physp),
 			     payload, sizeof(payload), IB_MAD_ATTR_SM_INFO,
 			     IB_SMINFO_ATTR_MOD_HANDOVER, FALSE,
 			     ib_port_info_get_m_key(&p_port->p_physp->port_info),
-			     CL_DISP_MSGID_NONE, &context);
+			     0, CL_DISP_MSGID_NONE, &context);
 	CL_PLOCK_RELEASE(sm->p_lock);
 
 	if (status != IB_SUCCESS)
@@ -1421,7 +1408,7 @@ repeat_discovery:
 	/* Reset tracking values in case limiting component got removed
 	 * from fabric. */
 	sm->p_subn->min_ca_mtu = IB_MAX_MTU;
-	sm->p_subn->min_ca_rate = IB_MAX_RATE;
+	sm->p_subn->min_ca_rate = IB_RATE_MAX;
 	sm->p_subn->min_data_vls = IB_MAX_NUM_VLS - 1;
 	sm->p_subn->min_sw_data_vls = IB_MAX_NUM_VLS - 1;
 
@@ -1690,9 +1677,12 @@ repeat_discovery:
 		osm_dump_all(sm->p_subn->p_osm);
 		state_mgr_up_msg(sm);
 
-		if (OSM_LOG_IS_ACTIVE_V2(sm->p_log, OSM_LOG_VERBOSE) ||
-		    sm->p_subn->opt.sa_db_dump)
-			osm_sa_db_file_dump(sm->p_subn->p_osm);
+		if ((OSM_LOG_IS_ACTIVE_V2(sm->p_log, OSM_LOG_VERBOSE) ||
+		    sm->p_subn->opt.sa_db_dump) &&
+		    !osm_sa_db_file_dump(sm->p_subn->p_osm))
+			osm_opensm_report_event(sm->p_subn->p_osm,
+						OSM_EVENT_ID_SA_DB_DUMPED,
+						NULL);
 	}
 
 	/*
@@ -1700,9 +1690,6 @@ repeat_discovery:
 	 */
 	cl_event_signal(&sm->subnet_up_event);
 
-	osm_opensm_report_event(sm->p_subn->p_osm, OSM_EVENT_ID_SUBNET_UP,
-				NULL);
-
 	/* if we got a signal to force heavy sweep or errors
 	 * in the middle of the sweep - try another sweep. */
 	if (sm->p_subn->force_heavy_sweep)
@@ -1712,6 +1699,9 @@ repeat_discovery:
 	osm_db_store(sm->p_subn->p_g2m, sm->p_subn->opt.fsync_high_avail_files);
 	osm_db_store(sm->p_subn->p_neighbor,
 		     sm->p_subn->opt.fsync_high_avail_files);
+
+	osm_opensm_report_event(sm->p_subn->p_osm, OSM_EVENT_ID_SUBNET_UP,
+				NULL);
 }
 
 static void do_process_mgrp_queue(osm_sm_t * sm)
