@@ -253,7 +253,16 @@ static int disable_port(osm_sm_t *sm, osm_physp_t *p)
 		if (!pi->base_lid) {
 			p_port = osm_get_port_by_guid(sm->p_subn,
 						      osm_physp_get_port_guid(p));
-			pi->base_lid = p_port->lid;
+			if (p_port)
+				pi->base_lid = p_port->lid;
+			else {
+				OSM_LOG(sm->p_log, OSM_LOG_ERROR,
+					"ERR 3804: Port 0x%" PRIx64
+					" not found, port set failed\n",
+					cl_ntoh64(osm_physp_get_port_guid(p)));
+				status = IB_ERROR;
+				goto EXIT;
+			}
 		}
 		pi->master_sm_base_lid = sm->p_subn->sm_base_lid;
 	}
@@ -263,6 +272,8 @@ static int disable_port(osm_sm_t *sm, osm_physp_t *p)
 			   cl_hton32(osm_physp_get_port_num(p)),
 			   FALSE, m_key,
 			   0, CL_DISP_MSGID_NONE, &context);
+
+EXIT:
 	return status;
 }
 
@@ -374,7 +385,7 @@ static void trap_rcv_process_request(IN osm_sm_t * sm,
 	boolean_t is_gsi = TRUE;
 	uint8_t port_num = 0;
 	boolean_t physp_change_trap = FALSE;
-	uint64_t event_wheel_timeout = OSM_DEFAULT_TRAP_SUPRESSION_TIMEOUT;
+	uint64_t event_wheel_timeout = OSM_DEFAULT_TRAP_SUPPRESSION_TIMEOUT;
 	boolean_t run_heavy_sweep = FALSE;
 	char buf[1024];
 	osm_dr_path_t *p_path;
@@ -493,7 +504,7 @@ static void trap_rcv_process_request(IN osm_sm_t * sm,
 						       trap_key);
 
 		/* Now we know how many times it provided this trap */
-		if (num_received > 10) {
+		if (num_received >= 10) {
 			if (print_num_received(num_received))
 				OSM_LOG(sm->p_log, OSM_LOG_VERBOSE,
 					"Received trap %u times consecutively\n",
@@ -529,7 +540,7 @@ static void trap_rcv_process_request(IN osm_sm_t * sm,
 					   NULL, NULL);
 
 		/* If was already registered do nothing more */
-		if (num_received > 10 && run_heavy_sweep == FALSE) {
+		if (num_received >= 10 && run_heavy_sweep == FALSE) {
 			if (print_num_received(num_received))
 				OSM_LOG(sm->p_log, OSM_LOG_VERBOSE,
 					"Ignoring noisy traps.\n");
