@@ -3,7 +3,7 @@
  * Copyright (c) 2002-2015 Mellanox Technologies LTD. All rights reserved.
  * Copyright (c) 1996-2003 Intel Corporation. All rights reserved.
  * Copyright (c) 2009-2015 ZIH, TU Dresden, Federal Republic of Germany. All rights reserved.
- * Copyright (C) 2012-2013 Tokyo Institute of Technology. All rights reserved.
+ * Copyright (C) 2012-2017 Tokyo Institute of Technology. All rights reserved.
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -48,6 +48,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <complib/cl_heap.h>
 #include <opensm/osm_file_ids.h>
 #define FILE_ID OSM_FILE_UCAST_DFSSSP_C
 #include <opensm/osm_ucast_mgr.h>
@@ -91,18 +92,13 @@ typedef struct vertex {
 	link_t *used_link;	/* link between the vertex discovered before and this vertex */
 	uint64_t distance;	/* distance from source to this vertex */
 	uint8_t state;
-	/* for the binary heap */
-	uint32_t heap_id;
+	/* for the d-ary heap */
+	size_t heap_index;
 	/* for LFT writing and debug */
 	osm_switch_t *sw;	/* selfpointer */
 	boolean_t dropped;	/* indicate dropped switches (w/ ucast cache) */
 } vertex_t;
 
-typedef struct binary_heap {
-	uint32_t size;		/* size of the heap */
-	vertex_t **nodes;	/* array with pointers to elements of the adj_list */
-} binary_heap_t;
-
 typedef struct vltable {
 	uint64_t num_lids;	/* size of the lids array */
 	uint16_t *lids;		/* sorted array of all lids in the subnet */
@@ -160,7 +156,7 @@ static inline void set_default_vertex(vertex_t * vertex)
 	vertex->used_link = NULL;
 	vertex->distance = 0;
 	vertex->state = UNDISCOVERED;
-	vertex->heap_id = 0;
+	vertex->heap_index = 0;
 	vertex->sw = NULL;
 	vertex->dropped = FALSE;
 }
@@ -177,166 +173,6 @@ static inline void set_default_cdg_node(cdg_node_t * node)
 	node->parent = NULL;
 }
 
-/**********************************************************************
- **********************************************************************/
-
-/************ helper functions for heap in dijkstra *******************
- **********************************************************************/
-/* returns true if element 1 is smaller than element 2 */
-static inline uint32_t heap_smaller(binary_heap_t * heap, uint32_t i,
-				    uint32_t j)
-{
-	return (heap->nodes[i]->distance < heap->nodes[j]->distance) ? 1 : 0;
-}
-
-/* swap two elements */
-static void heap_exchange(binary_heap_t * heap, uint32_t i, uint32_t j)
-{
-	uint32_t tmp_heap_id = 0;
-	vertex_t *tmp_node = NULL;
-
-	/* 1. swap the heap_id */
-	tmp_heap_id = heap->nodes[i]->heap_id;
-	heap->nodes[i]->heap_id = heap->nodes[j]->heap_id;
-	heap->nodes[j]->heap_id = tmp_heap_id;
-	/* 2. swap pointers */
-	tmp_node = heap->nodes[i];
-	heap->nodes[i] = heap->nodes[j];
-	heap->nodes[j] = tmp_node;
-}
-
-/* changes position of element with parent until children are bigger */
-static uint32_t heap_up(binary_heap_t * heap, uint32_t i)
-{
-	uint32_t curr = i, father = 0;
-
-	if (curr > 0) {
-		father = (curr - 1) >> 1;
-		while (heap_smaller(heap, curr, father)) {
-			heap_exchange(heap, curr, father);
-			/* try to go up when we arent already root */
-			curr = father;
-			if (curr > 0)
-				father = (curr - 1) >> 1;
-		}
-	}
-
-	return curr;
-}
-
-/* changes position of element with children until parent is smaller */
-static uint32_t heap_down(binary_heap_t * heap, uint32_t i)
-{
-	uint32_t curr = i;
-	uint32_t son1 = 0, son2 = 0, smaller_son = 0;
-	uint32_t exchanged = 0;
-
-	do {
-		son1 = ((curr + 1) << 1) - 1;
-		son2 = (curr + 1) << 1;
-		exchanged = 0;
-
-		/* exchange with smaller son */
-		if (son1 < heap->size && son2 < heap->size) {
-			if (heap_smaller(heap, son1, son2))
-				smaller_son = son1;
-			else
-				smaller_son = son2;
-		} else if (son1 < heap->size) {
-			/* only one son */
-			smaller_son = son1;
-		} else {
-			/* finished */
-			break;
-		}
-
-		/* only exchange when smaller */
-		if (heap_smaller(heap, smaller_son, curr)) {
-			heap_exchange(heap, curr, smaller_son);
-			exchanged = 1;
-			curr = smaller_son;
-		}
-	} while (exchanged);
-
-	return curr;
-}
-
-/* reheapify element */
-static inline void heap_heapify(binary_heap_t * heap, uint32_t i)
-{
-	heap_down(heap, heap_up(heap, i));
-}
-
-/* creates heap for graph */
-static int heap_create(vertex_t * adj_list, uint32_t adj_list_size,
-		       binary_heap_t ** binheap)
-{
-	binary_heap_t *heap = NULL;
-	uint32_t i = 0;
-
-	/* allocate the memory for the heap object */
-	heap = (binary_heap_t *) malloc(sizeof(binary_heap_t));
-	if (!heap)
-		return 1;
-
-	/* the heap size is equivalent to the size of the adj_list */
-	heap->size = adj_list_size;
-
-	/* allocate the pointer array, fill with the pointers to the elements of the adj_list and set the initial heap_id */
-	heap->nodes = (vertex_t **) malloc(heap->size * sizeof(vertex_t *));
-	if (!heap->nodes) {
-		free(heap);
-		return 1;
-	}
-	for (i = 0; i < heap->size; i++) {
-		heap->nodes[i] = &adj_list[i];
-		heap->nodes[i]->heap_id = i;
-	}
-
-	/* sort elements */
-	for (i = heap->size; i > 0; i--)
-		heap_down(heap, i - 1);
-
-	*binheap = heap;
-	return 0;
-}
-
-/* returns current minimum and removes it from heap */
-static vertex_t *heap_getmin(binary_heap_t * heap)
-{
-	vertex_t *min = NULL;
-
-	if (heap->size > 0)
-		min = heap->nodes[0];
-
-	if (min == NULL)
-		return min;
-
-	if (heap->size > 0) {
-		if (heap->size > 1) {
-			heap_exchange(heap, 0, heap->size - 1);
-			heap->size--;
-			heap_down(heap, 0);
-		} else {
-			heap->size--;
-		}
-	}
-
-	return min;
-}
-
-/* cleanup heap */
-static void heap_free(binary_heap_t * heap)
-{
-	if (heap) {
-		if (heap->nodes) {
-			free(heap->nodes);
-			heap->nodes = NULL;
-		}
-		free(heap);
-	}
-}
-
 /**********************************************************************
  **********************************************************************/
 
@@ -812,6 +648,9 @@ static int update_channel_dep_graph(cdg_node_t ** cdg_root,
 	while (remote_node && remote_node->sw) {
 		local_node = remote_node;
 		local_port = local_node->sw->new_lft[dlid];
+		/* sanity check: local_port must be set or routing is broken */
+		if (local_port == OSM_NO_PATH)
+			goto ERROR;
 		local_lid = cl_ntoh16(osm_node_get_base_lid(local_node, 0));
 		/* each port belonging to a switch has lmc==0 -> get_base_lid is fine
 		   (local/remote port in this function are always part of a switch)
@@ -961,6 +800,9 @@ static int remove_path_from_cdg(cdg_node_t ** cdg_root, osm_port_t * src_port,
 	while (remote_node && remote_node->sw) {
 		local_node = remote_node;
 		local_port = local_node->sw->new_lft[dlid];
+		/* sanity check: local_port must be set or routing is broken */
+		if (local_port == OSM_NO_PATH)
+			goto ERROR;
 		local_lid = cl_ntoh16(osm_node_get_base_lid(local_node, 0));
 
 		remote_node =
@@ -1028,7 +870,8 @@ static void add_sw_endports_to_order_list(osm_switch_t * sw,
 					  boolean_t add_guids)
 {
 	osm_port_t *port;
-	uint64_t port_guid;
+	ib_net64_t port_guid;
+	uint64_t sw_guid;
 	osm_physp_t *p;
 	int i;
 	boolean_t found;
@@ -1037,6 +880,17 @@ static void add_sw_endports_to_order_list(osm_switch_t * sw,
 		p = osm_node_get_physp_ptr(sw->p_node, i);
 		if (p && p->p_remote_physp && !p->p_remote_physp->p_node->sw) {
 			port_guid = p->p_remote_physp->port_guid;
+			/* check if link is healthy, otherwise ignore CA */
+			if (!osm_link_is_healthy(p)) {
+				sw_guid =
+				    cl_ntoh64(osm_node_get_node_guid
+					      (sw->p_node));
+				OSM_LOG(m->p_log, OSM_LOG_INFO,
+					"WRN AD40: ignoring CA due to unhealthy"
+					" link from switch 0x%016" PRIx64
+					" port %" PRIu8 " to CA 0x%016" PRIx64
+					"\n", sw_guid, i, cl_ntoh64(port_guid));
+			}
 			port = osm_get_port_by_guid(m->p_subn, port_guid);
 			if (!port)
 				continue;
@@ -1112,6 +966,7 @@ static int add_guid_to_map(void * cxt, uint64_t guid, char * p)
 {
 	cl_qmap_t *map = cxt;
 	name_map_item_t *item;
+	name_map_item_t *inserted_item;
 
 	item = malloc(sizeof(*item));
 	if (!item)
@@ -1119,7 +974,9 @@ static int add_guid_to_map(void * cxt, uint64_t guid, char * p)
 
 	item->guid = cl_hton64(guid);	/* internal: network byte order */
 	item->name = NULL;		/* name isn't needed */
-	cl_qmap_insert(map, item->guid, &item->item);
+	inserted_item = (name_map_item_t *) cl_qmap_insert(map, item->guid, &item->item);
+	if (inserted_item != item)
+                free(item);
 
 	return 0;
 }
@@ -1176,13 +1033,16 @@ static void dfsssp_print_graph(osm_ucast_mgr_t * p_mgr, vertex_t * adj_list,
 
 /* predefine, to use this in next function */
 static void dfsssp_context_destroy(void *context);
+static int dijkstra(osm_ucast_mgr_t * p_mgr, cl_heap_t * p_heap,
+		    vertex_t * adj_list, uint32_t adj_list_size,
+		    osm_port_t * port, uint16_t lid);
 
 /* traverse subnet to gather information about the connected switches */
 static int dfsssp_build_graph(void *context)
 {
 	dfsssp_context_t *dfsssp_ctx = (dfsssp_context_t *) context;
 	osm_ucast_mgr_t *p_mgr = (osm_ucast_mgr_t *) (dfsssp_ctx->p_mgr);
-
+	boolean_t has_fdr10 = (1 == p_mgr->p_subn->opt.fdr10) ? TRUE : FALSE;
 	cl_qmap_t *port_tbl = &p_mgr->p_subn->port_guid_tbl;	/* 1 management port per switch + 1 or 2 ports for each Hca */
 	osm_port_t *p_port = NULL;
 	cl_qmap_t *sw_tbl = &p_mgr->p_subn->sw_guid_tbl;
@@ -1190,13 +1050,15 @@ static int dfsssp_build_graph(void *context)
 	osm_switch_t *sw = NULL;
 	osm_node_t *remote_node = NULL;
 	uint8_t port = 0, remote_port = 0;
-	uint32_t i = 0, j = 0;
+	uint32_t i = 0, j = 0, err = 0, undiscov = 0, max_num_undiscov = 0;
 	uint64_t total_num_hca = 0;
 	vertex_t *adj_list = NULL;
 	osm_physp_t *p_physp = NULL;
 	link_t *link = NULL, *head = NULL;
 	uint32_t num_sw = 0, adj_list_size = 0;
 	uint8_t lmc = 0;
+	uint16_t sm_lid = 0;
+	cl_heap_t heap;
 
 	OSM_LOG_ENTER(p_mgr->p_log);
 	OSM_LOG(p_mgr->p_log, OSM_LOG_VERBOSE,
@@ -1208,6 +1070,9 @@ static int dfsssp_build_graph(void *context)
 	if (dfsssp_ctx->adj_list)
 		dfsssp_context_destroy(context);
 
+	/* construct the generic heap opject to use it in dijkstra */
+	cl_heap_construct(&heap);
+
 	num_sw = cl_qmap_count(sw_tbl);
 	adj_list_size = num_sw + 1;
 	/* allocate an adjazenz list (array), 0. element is reserved for the source (Hca) in the routing algo, others are switches */
@@ -1254,7 +1119,6 @@ static int dfsssp_build_graph(void *context)
 		if (!link) {
 			OSM_LOG(p_mgr->p_log, OSM_LOG_ERROR,
 				"ERR AD03: cannot allocate memory for a link\n");
-			dfsssp_context_destroy(context);
 			goto ERROR;
 		}
 		head = link;
@@ -1283,6 +1147,19 @@ static int dfsssp_build_graph(void *context)
 				adj_list[i].num_hca += (1 << lmc);
 				continue;
 			}
+			/* filter out throttled links to improve performance */
+			if (p_mgr->p_subn->opt.avoid_throttled_links &&
+			    osm_link_is_throttled(p_physp, has_fdr10)) {
+				OSM_LOG(p_mgr->p_log, OSM_LOG_INFO,
+					"Detected and ignoring throttled link:"
+					" 0x%" PRIx64 "/P%" PRIu8
+					" <--> 0x%" PRIx64 "/P%" PRIu8 "\n",
+					cl_ntoh64(osm_node_get_node_guid(sw->p_node)),
+					port,
+					cl_ntoh64(osm_node_get_node_guid(remote_node)),
+					remote_port);
+				continue;
+			}
 			OSM_LOG(p_mgr->p_log, OSM_LOG_DEBUG,
 				"Node 0x%" PRIx64 ", remote node 0x%" PRIx64
 				", port %" PRIu8 ", remote port %" PRIu8 "\n",
@@ -1294,7 +1171,6 @@ static int dfsssp_build_graph(void *context)
 			if (!link->next) {
 				OSM_LOG(p_mgr->p_log, OSM_LOG_ERROR,
 					"ERR AD08: cannot allocate memory for a link\n");
-				dfsssp_context_destroy(context);
 				while (head) {
 					link = head;
 					head = head->next;
@@ -1328,6 +1204,32 @@ static int dfsssp_build_graph(void *context)
 			link = link->next;
 		}
 	}
+
+	/* do one dry run to determine connectivity issues */
+	sm_lid = p_mgr->p_subn->master_sm_base_lid;
+	p_port = osm_get_port_by_lid(p_mgr->p_subn, sm_lid);
+	err = dijkstra(p_mgr, &heap, adj_list, adj_list_size, p_port, sm_lid);
+	if (err) {
+		goto ERROR;
+	} else {
+		/* if sm is running on a switch, then dijkstra doesn't
+		   initialize the used_link for this switch
+		 */
+		if (osm_node_get_type(p_port->p_node) != IB_NODE_TYPE_CA)
+			max_num_undiscov = 1;
+		for (i = 1; i < adj_list_size; i++)
+			undiscov += (adj_list[i].used_link) ? 0 : 1;
+		if (max_num_undiscov < undiscov) {
+			OSM_LOG(p_mgr->p_log, OSM_LOG_ERROR,
+				"ERR AD0C: unsupported network state (detached"
+				" and inaccessible switches found; gracefully"
+				" shutdown this routing engine)\n");
+			goto ERROR;
+		}
+	}
+	/* delete the heap which is not needed anymore */
+	cl_heap_destroy(&heap);
+
 	/* print the discovered graph */
 	if (OSM_LOG_IS_ACTIVE_V2(p_mgr->p_log, OSM_LOG_DEBUG))
 		dfsssp_print_graph(p_mgr, adj_list, adj_list_size);
@@ -1336,6 +1238,9 @@ static int dfsssp_build_graph(void *context)
 	return 0;
 
 ERROR:
+	if (cl_is_heap_inited(&heap))
+		cl_heap_destroy(&heap);
+	dfsssp_context_destroy(context);
 	return -1;
 }
 
@@ -1376,9 +1281,18 @@ static void print_routes(osm_ucast_mgr_t * p_mgr, vertex_t * adj_list,
 	}
 }
 
+/* callback function for the cl_heap to update the index */
+static void apply_index_update(const void * context, const size_t new_index)
+{
+	vertex_t *heap_elem = (vertex_t *) context;
+	if (heap_elem)
+		heap_elem->heap_index = new_index;
+}
+
 /* dijkstra step from one source to all switches in the df-/sssp graph */
-static int dijkstra(osm_ucast_mgr_t * p_mgr, vertex_t * adj_list,
-		    uint32_t adj_list_size, osm_port_t * port, uint16_t lid)
+static int dijkstra(osm_ucast_mgr_t * p_mgr, cl_heap_t * p_heap,
+		    vertex_t * adj_list, uint32_t adj_list_size,
+		    osm_port_t * port, uint16_t lid)
 {
 	uint32_t i = 0, j = 0, index = 0;
 	osm_node_t *remote_node = NULL;
@@ -1386,17 +1300,34 @@ static int dijkstra(osm_ucast_mgr_t * p_mgr, vertex_t * adj_list,
 	vertex_t *current = NULL;
 	link_t *link = NULL;
 	uint64_t guid = 0;
-	binary_heap_t *heap = NULL;
-	int err = 0;
+	cl_status_t ret = CL_SUCCESS;
 
 	OSM_LOG_ENTER(p_mgr->p_log);
 
+	/* build an 4-ary heap to find the node with minimum distance */
+	if (!cl_is_heap_inited(p_heap))
+		ret = cl_heap_init(p_heap, adj_list_size, 4,
+				   &apply_index_update, NULL);
+	else
+		ret = cl_heap_resize(p_heap, adj_list_size);
+	if (ret != CL_SUCCESS) {
+		OSM_LOG(p_mgr->p_log, OSM_LOG_ERROR,
+			"ERR AD09: cannot allocate memory or resize heap\n");
+		return ret;
+	}
+
 	/* reset all switches for new round with a new source for dijkstra */
 	for (i = 1; i < adj_list_size; i++) {
 		adj_list[i].hops = 0;
 		adj_list[i].used_link = NULL;
 		adj_list[i].distance = INF;
 		adj_list[i].state = UNDISCOVERED;
+		ret = cl_heap_insert(p_heap, INF, &adj_list[i]);
+		if (ret != CL_SUCCESS) {
+			OSM_LOG(p_mgr->p_log, OSM_LOG_ERROR,
+				"ERR AD11: cl_heap_insert failed\n");
+			return ret;
+		}
 	}
 
 	/* if behind port is a Hca -> set adj_list[0] */
@@ -1450,6 +1381,20 @@ static int dijkstra(osm_ucast_mgr_t * p_mgr, vertex_t * adj_list,
 					}
 				}
 			}
+		} else {
+			/* if link is unhealthy then there's a severe issue */
+			OSM_LOG(p_mgr->p_log, OSM_LOG_ERROR,
+				"ERR AD0B: unsupported network state (CA with"
+				" unhealthy link state discovered; should have"
+				" been filtered out before already; gracefully"
+				" shutdown this routing engine)\n");
+			return 1;
+		}
+		ret = cl_heap_insert(p_heap, INF, &adj_list[0]);
+		if (ret != CL_SUCCESS) {
+			OSM_LOG(p_mgr->p_log, OSM_LOG_ERROR,
+				"ERR AD13: cl_heap_insert failed\n");
+			return ret;
 		}
 		/* if behind port is a switch -> search switch in adj_list */
 	} else {
@@ -1472,19 +1417,15 @@ static int dijkstra(osm_ucast_mgr_t * p_mgr, vertex_t * adj_list,
 	adj_list[index].distance = 0;
 	adj_list[index].state = DISCOVERED;
 	adj_list[index].hops = 0;	/* the source has hop count = 0 */
-
-	/* create a heap to find (efficient) the node with the smallest distance */
-	if (osm_node_get_type(port->p_node) == IB_NODE_TYPE_CA)
-		err = heap_create(adj_list, adj_list_size, &heap);
-	else
-		err = heap_create(&adj_list[1], adj_list_size - 1, &heap);
-	if (err) {
+	ret = cl_heap_modify_key(p_heap, adj_list[index].distance,
+				 adj_list[index].heap_index);
+	if (ret != CL_SUCCESS) {
 		OSM_LOG(p_mgr->p_log, OSM_LOG_ERROR,
-			"ERR AD09: cannot allocate memory for heap or heap->node in heap_create(...)\n");
-		return err;
+			"ERR AD10: index out of bounds in cl_heap_modify_key\n");
+		return ret;
 	}
 
-	current = heap_getmin(heap);
+	current = (vertex_t *) cl_heap_extract_root(p_heap);
 	while (current) {
 		current->state = DISCOVERED;
 		if (current->used_link)	/* increment the number of hops to the source for each new node */
@@ -1499,17 +1440,20 @@ static int dijkstra(osm_ucast_mgr_t * p_mgr, vertex_t * adj_list,
 				adj_list[link->to].used_link = link;
 				adj_list[link->to].distance =
 				    current->distance + link->weight;
-				heap_heapify(heap, adj_list[link->to].heap_id);
+				ret = cl_heap_modify_key(p_heap,
+							 adj_list[link->to].distance,
+							 adj_list[link->to].heap_index);
+				if (ret != CL_SUCCESS) {
+					OSM_LOG(p_mgr->p_log, OSM_LOG_ERROR,
+						"ERR AD12: index out of bounds in cl_heap_modify_key\n");
+					return ret;
+				}
 			}
 		}
 
-		current = heap_getmin(heap);
+		current = (vertex_t *) cl_heap_extract_root(p_heap);
 	}
 
-	/* destroy the heap */
-	heap_free(heap);
-	heap = NULL;
-
 	OSM_LOG_EXIT(p_mgr->p_log);
 	return 0;
 }
@@ -2233,6 +2177,7 @@ static int dfsssp_do_dijkstra_routing(void *context)
 	osm_ucast_mgr_t *p_mgr = (osm_ucast_mgr_t *) dfsssp_ctx->p_mgr;
 	vertex_t *adj_list = (vertex_t *) dfsssp_ctx->adj_list;
 	uint32_t adj_list_size = dfsssp_ctx->adj_list_size;
+	cl_heap_t heap;
 
 	vertex_t **sw_list = NULL;
 	uint32_t sw_list_size = 0;
@@ -2277,6 +2222,9 @@ static int dfsssp_do_dijkstra_routing(void *context)
 		}
 	}
 
+	/* construct the generic heap opject to use it in dijkstra */
+	cl_heap_construct(&heap);
+
 	/* we need an intermediate array of pointers to switches in adj_list;
 	   this array will be sorted in respect to num_hca (descending)
 	 */
@@ -2443,7 +2391,8 @@ static int dfsssp_do_dijkstra_routing(void *context)
 		for (lid = min_lid_ho; lid <= max_lid_ho; lid++) {
 			/* do dijkstra from this Hca/LID/SP0 to each switch */
 			err =
-			    dijkstra(p_mgr, adj_list, adj_list_size, port, lid);
+			    dijkstra(p_mgr, &heap, adj_list, adj_list_size,
+				     port, lid);
 			if (err)
 				goto ERROR;
 			if (OSM_LOG_IS_ACTIVE_V2(p_mgr->p_log, OSM_LOG_DEBUG))
@@ -2483,6 +2432,9 @@ static int dfsssp_do_dijkstra_routing(void *context)
 	/* list not needed after the dijkstra steps and deadlock removal */
 	cl_qlist_remove_all(&p_mgr->port_order_list);
 
+	/* delete the heap which is not needed anymore */
+	cl_heap_destroy(&heap);
+
 	/* print the new_lft for each switch after routing is done */
 	if (OSM_LOG_IS_ACTIVE_V2(p_mgr->p_log, OSM_LOG_DEBUG)) {
 		for (item = cl_qmap_head(sw_tbl); item != cl_qmap_end(sw_tbl);
@@ -2515,6 +2467,8 @@ ERROR:
 		destroy_guid_map(&io_tbl);
 	if (sw_list)
 		free(sw_list);
+	if (cl_is_heap_inited(&heap))
+		cl_heap_destroy(&heap);
 	return -1;
 }
 
@@ -2538,6 +2492,7 @@ static ib_api_status_t dfsssp_do_mcast_routing(void * context,
 	uint32_t err = 0, num_ports = 0, i = 0;
 	ib_net64_t guid = 0;
 	ib_api_status_t status = IB_SUCCESS;
+	cl_heap_t heap;
 
 	OSM_LOG_ENTER(sm->p_log);
 
@@ -2584,6 +2539,9 @@ static ib_api_status_t dfsssp_do_mcast_routing(void * context,
 		}
 	}
 
+	/* construct the generic heap opject to use it in dijkstra */
+	cl_heap_construct(&heap);
+
 	/* create a map and a list of all ports which are member in the mcast
 	   group; map for searching elements and list for iteration
 	 */
@@ -2628,7 +2586,7 @@ static ib_api_status_t dfsssp_do_mcast_routing(void * context,
 	 */
 	lid = osm_node_get_base_lid(root_sw->p_node, 0);
 	port = osm_get_port_by_lid(sm->p_subn, lid);
-	err = dijkstra(p_mgr, adj_list, adj_list_size, port, lid);
+	err = dijkstra(p_mgr, &heap, adj_list, adj_list_size, port, lid);
 	if (err) {
 		OSM_LOG(sm->p_log, OSM_LOG_ERROR, "ERR AD52: "
 			"Dijkstra step for mcast failed for group 0x%X\n",
@@ -2654,6 +2612,8 @@ static ib_api_status_t dfsssp_do_mcast_routing(void * context,
 	}
 
 Exit:
+	if (cl_is_heap_inited(&heap))
+		cl_heap_destroy(&heap);
 	reset_mgrp_membership(adj_list, adj_list_size);
 	osm_mcast_drop_port_list(&mcastgrp_port_list);
 	OSM_LOG_EXIT(sm->p_log);
