@@ -61,7 +61,7 @@
 #define OSMT_DEFAULT_TRAP_WAIT_TIMEOUT_SEC 10
 #define INVALID_GUID (0xFFFFFFFFFFFFFFFFULL)
 
-boolean_t osmt_is_debug(void)
+static boolean_t osmt_is_debug(void)
 {
 #if defined( _DEBUG_ )
 	return TRUE;
@@ -70,7 +70,7 @@ boolean_t osmt_is_debug(void)
 #endif				/* defined( _DEBUG_ ) */
 }
 
-void show_usage()
+static void show_usage(void)
 {
 	printf
 	    ("\n------- osmtest - Usage and options ----------------------\n");
@@ -234,7 +234,7 @@ static void print_all_guids(IN osmtest_t * p_osmt)
 		       cl_hton64(attr_array[i].port_guid));
 }
 
-ib_net64_t get_port_guid(IN osmtest_t * p_osmt, uint64_t port_guid)
+static ib_net64_t get_port_guid(IN osmtest_t * p_osmt, uint64_t port_guid)
 {
 	ib_api_status_t status;
 	uint32_t num_ports = MAX_LOCAL_IBPORTS;
@@ -280,7 +280,7 @@ int main(int argc, char *argv[])
 	uint32_t log_flags = OSM_LOG_ERROR | OSM_LOG_INFO;
 	int32_t vendor_debug = 0;
 	char flow_name[64];
-	uint32_t next_option;
+	int next_option;
 	const char *const short_option = "f:l:m:M:d:g:s:t:i:pcvGVh";
 
 	/*
@@ -582,7 +582,11 @@ int main(int argc, char *argv[])
 	if (vendor_debug)
 		osm_vendor_set_debug(osm_test.p_vendor, vendor_debug);
 
-	complib_init();
+	if (complib_init_v2() != CL_SUCCESS) {
+		printf("\ncomplib_init_v2 failed\n");
+		status = IB_ERROR;
+		goto Exit;
+	}
 
 	status = osmtest_init(&osm_test, &opt, (osm_log_level_t) log_flags);
 	if (status != IB_SUCCESS) {
